from django.contrib import admin
from django.urls import path, include

from ..urls import urlpatterns as log_viewer_urlpatterns

urlpatterns = [
    path("admin/", admin.site.urls),
    path("logs/", include("log_viewer.urls")),
]
urlpatterns += log_viewer_urlpatterns



import types
import tempfile
from itertools import islice

from django.urls import reverse
from django.test import TestCase, Client
from django.contrib.auth import get_user_model

from ..utils import get_log_files, readlines_reverse

User = get_user_model()


class TestLogsViewer(TestCase):
    def setUp(self):
        self.user = User.objects.create_user(
            username="john",
            password="Foobar@1234",
            is_active=True,
            is_staff=True,
            is_superuser=True,
        )
        self.client_login = Client()
        self.client_login.force_login(self.user)
        self.client_anon = Client()
        self.url = reverse("log_file_view")

    def test_redirection(self):
        response = self.client_anon.get(self.url)
        self.assertEqual(response.status_code, 302)

    def test_logs_viewer(self):
        response = self.client_login.get(self.url)
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, "Django Log Viewer")
        self.assertTemplateUsed(response, "log_viewer/logfile_viewer.html")

    def test_get_log_files(self):
        with tempfile.TemporaryDirectory() as log_dir:
            open("%s/default.log" % log_dir, "a").close()
            result = get_log_files(log_dir, 1, 1)
            self.assertEqual(
                result,
                {
                    "logs": {
                        "": ["default.log"],
                    },
                    "next_page_files": 2,
                    "last_files": True,
                },
            )

    def test_readlines_reverse(self):
        with tempfile.TemporaryDirectory() as log_dir:
            with open("%s/default.log" % log_dir, "w") as qfile:
                qfile.write(
                    "[WARNING] 2022-08-26 13:02:46,070 django.request: Not Found: /404/"
                )
                qfile.write(
                    "[ERROR] 2022-08-26 13:02:50,962 django.request: Internal Server Error: /admin/"
                )
                qfile.write(
                    "[INFO] 2022-08-26 13:02:36,604 django.utils.autoreload: Watching for file changes"
                )
                qfile.close()

            with open(
                "%s/default.log" % log_dir, encoding="utf8", errors="ignore"
            ) as qfile:
                lines = readlines_reverse(qfile, exclude=None)
                lines_string = " ".join(map(str, lines))
                self.assertTrue(isinstance(lines, types.GeneratorType))
                self.assertIn("[WARNING]", lines_string)
                self.assertIn("[ERROR]", lines_string)
                self.assertIn("[INFO]", lines_string)



import django

if django.VERSION >= (2, 0):
    from django.urls import re_path
else:
    from django.conf.urls import url as re_path
from .views import log_json, log_viewer, log_download

app_name = "log_viewer"

urlpatterns = [
    re_path(
        r"^json/(?P<file_name>[\.\w-]*)/(?P<page>[0-9]+)$",
        log_json,
        name="log_json_view",
    ),
    re_path(
        r"^json/(?P<file_name>[\.\w-]*)$",
        log_json,
        name="log_json_view",
    ),
    re_path(
        r"^download/single-file/$",
        log_download,
        name="log_download_file_view",
    ),
    re_path(
        r"^download.zip$",
        log_download,
        name="log_download_zip_view",
    ),
    re_path(
        r"^$",
        log_viewer,
        name="log_file_view",
    ),
]



from django.conf import settings

LOG_VIEWER_FILES = getattr(settings, "LOG_VIEWER_FILES", [])
LOG_VIEWER_FILES_PATTERN = getattr(settings, "LOG_VIEWER_FILES_PATTERN", "*.log*")
LOG_VIEWER_FILES_DIR = getattr(settings, "LOG_VIEWER_FILES_DIR", "logs/")
LOG_VIEWER_PAGE_LENGTH = getattr(settings, "LOG_VIEWER_PAGE_LENGTH", 25)
LOG_VIEWER_MAX_READ_LINES = getattr(settings, "LOG_VIEWER_MAX_READ_LINES", 1000)
LOG_VIEWER_FILE_LIST_TITLE = getattr(settings, "LOG_VIEWER_FILE_LIST_TITLE", None)
LOG_VIEWER_FILE_LIST_STYLES = getattr(settings, "LOG_VIEWER_FILE_LIST_STYLES", None)
LOG_VIEWER_FILE_LIST_MAX_ITEMS_PER_PAGE = getattr(
    settings, "LOG_VIEWER_FILE_LIST_MAX_ITEMS_PER_PAGE", 25
)
LOG_VIEWER_PATTERNS = getattr(
    settings,
    "LOG_VIEWER_PATTERNS",
    ["[INFO]", "[DEBUG]", "[WARNING]", "[ERROR]", "[CRITICAL]"],
)
LOG_VIEWER_EXCLUDE_TEXT_PATTERN = getattr(
    settings, "LOG_VIEWER_EXCLUDE_TEXT_PATTERN", None
)



import os
import re
from fnmatch import fnmatch
from django.http import JsonResponse

from log_viewer import settings


def get_log_files(directory, max_items_per_page, current_page):
    """
    Function to get the logs file names.
    :param `directory` is string of base logs path.
    :param `max_items_per_page` is int max items to show per-page.
    :param `current_page` is int of current page.
    :return dict of logs files data.
    >>> get_log_files("logs", 2, 1)
    {
      "logs": {
        "": [
          "default.log",
          "request_debug.log",
          "request_error.log"
        ]
      },
      "next_page_files": 2,
      "last_files": false
    }
    >>>
    """
    result = {}
    all_log_files = []
    for root, _, files in os.walk(directory):
        all_files = list(filter(lambda x: x.find("~") == -1, files))

        all_log_files.extend(
            list(filter(lambda x: x in settings.LOG_VIEWER_FILES, all_files))
        )
        all_log_files.extend(
            [x for x in all_files if fnmatch(x, settings.LOG_VIEWER_FILES_PATTERN)]
        )
        log_dir = os.path.relpath(root, directory)
        if log_dir == ".":
            log_dir = ""

        result["logs"] = {log_dir: list(set(all_log_files))}
        result["next_page_files"] = current_page + 1
        result["last_files"] = (
            all_log_files.__len__() <= current_page * max_items_per_page
        )

    return result


def readlines_reverse(qfile, exclude=None):
    """
    Read file lines from bottom to top
    :param `qfile` is queue read file, i.e: with open(..., 'r') as qfile:
    :param `exclude` is string regex expression to exclude the log from line.
    :return yield string
    """
    # support custom patterns (fixed issue #4)
    patterns = settings.LOG_VIEWER_PATTERNS
    reversed_patterns = [x[::-1] for x in patterns]

    qfile.seek(0, os.SEEK_END)
    position = qfile.tell()
    line = ""

    while position >= 0:
        qfile.seek(position)
        next_char = qfile.read(1)

        # original
        # exclude = ""  # in param
        # if next_char == "\n" and line and line[-1] == '[':
        #     if exclude in line[::-1]:
        #         line = ''
        #     else:
        #         yield line[::-1]
        #         line = ''
        # else:
        #     line += next_char

        # modified
        if next_char == "\n" and line:
            if any([line.endswith(p) for p in reversed_patterns]):
                if exclude and re.search(exclude, line[::-1]).group(0):
                    line = ""
                else:
                    yield line[::-1]
                    line = ""
        else:
            line += next_char
        position -= 1
    yield line[::-1]


class JSONResponseMixin:
    """
    A mixin that can be used to render a JSON response.
    """

    def render_to_json_response(self, context, **response_kwargs):
        """
        Returns a JSON response, transforming 'context' to make the payload.
        """
        return JsonResponse(self.get_data(context), **response_kwargs)

    def get_data(self, context):
        """
        Returns an object that will be serialized as JSON by json.dumps().
        """
        # Note: This is *EXTREMELY* naive; in reality, you'll need
        # to do much more complex handling to ensure that arbitrary
        # objects -- such as Django model instances or querysets
        # -- can be serialized as JSON.
        return context



import os
import zipfile
from io import BytesIO
from itertools import islice

from django.http import HttpResponse, Http404
from django.views.generic import TemplateView as _TemplateView
from django.contrib.auth.decorators import login_required, user_passes_test
from django.contrib.admin.utils import quote, unquote
from django.utils.decorators import method_decorator
from django.utils.functional import SimpleLazyObject
from django.utils.timezone import localtime, now
from django.conf import settings as django_settings

from log_viewer import settings
from log_viewer.utils import get_log_files, readlines_reverse, JSONResponseMixin


class TemplateView(_TemplateView):
    @method_decorator(login_required)
    @method_decorator(user_passes_test(lambda u: u.is_superuser))
    def dispatch(self, *args, **kwargs):
        return super(TemplateView, self).dispatch(*args, **kwargs)


class LogJsonView(JSONResponseMixin, TemplateView):
    def get_log_json(self, original_context={}):
        context = {}
        page = original_context.get("page", 1)
        file_name = original_context.get("file_name", "")

        # Clean the `file_name` to avoid relative paths.
        file_name = unquote(file_name).replace("/..", "").replace("..", "")
        page = int(page)
        current_file = file_name

        lines_per_page = settings.LOG_VIEWER_MAX_READ_LINES
        context["original_file_name"] = file_name
        context["next_page"] = page + 1
        context["log_files"] = []

        log_file_data = get_log_files(
            settings.LOG_VIEWER_FILES_DIR,
            settings.LOG_VIEWER_FILE_LIST_MAX_ITEMS_PER_PAGE,
            1,
        )
        context["next_page_files"] = log_file_data["next_page_files"]
        context["last_files"] = log_file_data["last_files"]

        for log_dir, log_files in log_file_data["logs"].items():
            for log_file in log_files:
                display = os.path.join(log_dir, log_file)
                uri = os.path.join(settings.LOG_VIEWER_FILES_DIR, display)

                context["log_files"].append(
                    {
                        quote(display): {
                            "uri": uri,
                            "display": display,
                        }
                    }
                )

        if file_name:
            try:
                file_log = os.path.join(settings.LOG_VIEWER_FILES_DIR, file_name)
                with open(file_log, encoding="utf8", errors="ignore") as file:
                    next_lines = list(
                        islice(
                            readlines_reverse(
                                file, exclude=settings.LOG_VIEWER_EXCLUDE_TEXT_PATTERN
                            ),
                            (page - 1) * lines_per_page,
                            page * lines_per_page,
                        )
                    )

                    if len(next_lines) < lines_per_page:
                        context["last"] = True
                    else:
                        context["last"] = False
                    context["logs"] = next_lines
                    context["current_file"] = current_file
                    context["file"] = file

            except Exception as error:
                print(error)
                pass
        else:
            context["last"] = True

        if len(context["log_files"]) > 0:
            context["log_files"] = sorted(
                context["log_files"], key=lambda x: sorted(x.items())
            )

        return context

    def render_to_response(self, context, **response_kwargs):

        # to support Django 3.1.* (fixed issue #6)
        file_name = context.get("file_name")
        if isinstance(file_name, SimpleLazyObject):
            context = context["view"].kwargs

        log_json = self.get_log_json(context)

        if "file" in log_json:
            log_json["file"] = log_json["file"].name

        if "view" in context:
            del context["view"]

        context.update(**log_json)
        return self.render_to_json_response(context, **response_kwargs)


class LogDownloadView(TemplateView):
    def render_to_response(self, context, **response_kwargs):
        # file_name = context.get('file_name', None)
        file_name = self.request.GET.get("file_name", None)
        log_file_result = get_log_files(
            settings.LOG_VIEWER_FILES_DIR,
            settings.LOG_VIEWER_FILE_LIST_MAX_ITEMS_PER_PAGE,
            1,
        )["logs"]

        if file_name:
            file_path = unquote(file_name)
            uri = os.path.join(settings.LOG_VIEWER_FILES_DIR, file_path)

            log_dir = os.path.dirname(file_path)
            log_file = os.path.basename(file_path)

            if log_file in log_file_result.get(log_dir, []):
                with open(uri, "rb") as f:
                    buffer = f.read()
                resp = HttpResponse(buffer, content_type="plain/text")
                resp["Content-Disposition"] = "attachment; filename=%s" % file_name
                return resp
            else:
                raise Http404

        else:
            generation_time = localtime() if django_settings.USE_TZ else now()
            zip_filename = "log_%s.zip" % generation_time.strftime("%Y%m%dT%H%M%S")
            zip_buffer = BytesIO()

            with zipfile.ZipFile(
                zip_buffer, "a", zipfile.ZIP_DEFLATED, False
            ) as zip_file:
                for log_dir, log_files in log_file_result.items():
                    for log_file in log_files:
                        display = os.path.join(log_dir, log_file)
                        uri = os.path.join(settings.LOG_VIEWER_FILES_DIR, display)

                        with open(uri, "r") as f:
                            zip_file.writestr("%s" % display, f.read())

            zip_buffer.seek(0)
            resp = HttpResponse(zip_buffer, content_type="application/zip")
            resp["Content-Disposition"] = "attachment; filename=%s" % zip_filename
            return resp


class LogViewerView(TemplateView):
    """
    LogViewerView class
    :template_name: Name of the HTML template used to render the log files
    """

    template_name = "log_viewer/logfile_viewer.html"

    def get_context_data(self, file_name=None, page=1, **kwargs):
        """
        Read and return log files to be showed in admin page

        :param file_name: log file name
        :param page: log viewer page
        """
        context = super(LogViewerView, self).get_context_data(**kwargs)
        context["custom_file_list_title"] = settings.LOG_VIEWER_FILE_LIST_TITLE
        context["custom_style_file"] = settings.LOG_VIEWER_FILE_LIST_STYLES
        context["page_length"] = settings.LOG_VIEWER_PAGE_LENGTH
        context["files_per_page"] = settings.LOG_VIEWER_FILE_LIST_MAX_ITEMS_PER_PAGE
        return context


log_json = LogJsonView.as_view()
log_viewer = LogViewerView.as_view()
log_download = LogDownloadView.as_view()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from setuptools import setup, find_packages

__version__ = "1.1.7"

setup(
    name="django-log-viewer",
    version=__version__,
    packages=find_packages(exclude=["*demo"]),
    include_package_data=True,
    zip_safe=False,
    description="Django log viewer",
    url="https://github.com/agusmakmun/django-log-viewer",
    download_url="https://github.com/agusmakmun/django-log-viewer/tarball/v%s"
    % __version__,
    keywords=["django log viewer"],
    long_description=open("README.rst").read(),
    license="MIT",
    author="Agus Makmun (Summon Agus)",
    author_email="summon.agus@gmail.com",
    classifiers=[
        "Framework :: Django",
        "Framework :: Django :: 1.8",
        "Framework :: Django :: 1.9",
        "Framework :: Django :: 1.10",
        "Framework :: Django :: 1.11",
        "Framework :: Django :: 2.0",
        "Framework :: Django :: 3.0",
        "Framework :: Django :: 3.1",
        "Framework :: Django :: 4.0",
        "Intended Audience :: Developers",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3.3",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.9",
        "Development Status :: 5 - Production/Stable",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "License :: OSI Approved :: MIT License",
        "Environment :: Web Environment",
    ],
)
